package hu.elte.math;

/**
 * This class represents a two-dimensional matrix consisting of {@code int}s as its elements.
 * 
 * @author Istvan Neuwirth
 *
 */
public class Matrix {

    private final int[][] elements;
    private final int height;
    private final int width;

    /**
     * Constructs a two-dimensinal matrix based on its height and width.
     * Both height and width must be equal of greater than 1.
     * 
     * @throws IllegalArgumentException if height or width is less than 1.
     * 
     * @param height the height of the matrix
     * @param width the width of the matrix
     */
    public Matrix(int height, int width) {
        if (height < 1) {
            throw new IllegalArgumentException("invalid height: " + height);
        }
        if (width < 1) {
            throw new IllegalArgumentException("invalid width: " + width);
        }
        this.elements = new int[height][width];
        this.width = width;
        this.height = height;
    }

    /**
     * Adds another matrix to this instance. The values of the other matrix will not
     * be affected. The matrices must be compatible with each other so that both
     * their height and width values must be the same.
     * 
     * @throws IllegalArgumentException if the matrices are not compatible
     * 
     * @param other the other matrix to be added
     */
    public void add(Matrix other) {
        if (height != other.height) {
            throw new IllegalArgumentException("matrices are incompatible, height values are different (" + height + " and " + other.height + ")");
        }
        if (width != other.width) {
            throw new IllegalArgumentException("matrices are incompatible, width values are different (" + width + " and " + other.width + ")");
        }
        for (int i = 0; i < height; i++) {
            for (int j = 0; j < width; j++) {
                elements[i][j] += other.elements[i][j];
            }
        }
    }

    /**
     * Returns with the element of the certain position.
     * 
     * @param row the row of the element, indexing starts with 0
     * @param column the column of the element, indexing starts with 0
     * @return the element on position {@code row} and {@code column}
     */
    public int getElement(int row, int column) {
        return elements[row][column];
    }

    
    public void setElement(int element, int row, int column) {
        elements[row][column] = element;
    }

    /**
     * Returns the width of this matrix.
     * @return the width of this matrix
     */
    public int getWidth() {
        return width;
    }

    /**
     * Returns the height of this matrix.
     * @return the height of this matrix
     */
    public int getHeight() {
        return height;
    }

    /**
     * <p>Creates a {@link String} representation of this matrix. The representation
     * does not contain any line ending character.</p>
     * 
     * <p>For a matrix A</p>
     * <pre>
     * | 1 | 2 |
     * | 3 | 4 |
     * </pre>
     * <p>the generated representation is {@code "1 2 | 3 4"}</p>
     * 
     * @return the representation of this matrix
     */
    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder();
        for (int i = 0; i < height; i++) {
            for (int j = 0; j < width; j++) {
                sb.append(elements[i][j]);
                if (j < width - 1) {
                    sb.append(' ');
                }
            }
            if (i < height - 1) {
                sb.append(" | ");
            }
        }
        return sb.toString();
    }

}